@@ -5,9 +5,9 @@
 ###
 ### Plot-method(s) for fitted hhh4() models
 ###
-### Copyright (C) 2010-2012 Michaela Paul, 2012-2019 Sebastian Meyer
-### $Revision: 2351 $
-### $Date: 2019-02-20 09:25:44 +0100 (Wed, 20. Feb 2019) $
+### Copyright (C) 2010-2012 Michaela Paul, 2012-2020 Sebastian Meyer
+### $Revision: 2511 $
+### $Date: 2020-01-10 10:08:18 +0100 (Fri, 10. Jan 2020) $
 ################################################################################
 
 
@@ -273,12 +273,13 @@ plotComponentPolygons <- function (x, y, col = 1:6, border = col, add = FALSE)
 
 plotHHH4_maps <- function (x,
     which = c("mean", "endemic", "epi.own", "epi.neighbours"),
-    prop = FALSE, main = which, zmax = NULL,
-    col.regions = hcl.colors(10),
+    prop = FALSE, main = which, zmax = NULL, col.regions = NULL,
     labels = FALSE, sp.layout = NULL, ...,
     map = x$stsObj@map, meanHHH = NULL)
 {
     which <- match.arg(which, several.ok = TRUE)
+    if (is.null(col.regions))
+        col.regions <- .hcl.colors(10)
 
     ## extract district-specific mean components
     if (is.null(meanHHH)) {
@@ -328,9 +329,15 @@ plotHHH4_maps <- function (x,
     ## produce maps
     grobs <- mapply(
         FUN = function (zcol, main, zmax)
+        if (is.na(zmax)) { # automatic color breaks over range of values
+            spplot(map, zcol = zcol, main = main,
+                   cuts = length(col.regions) - 1L,
+                   col.regions = col.regions, sp.layout = sp.layout, ...)
+        } else { # breakpoints from 0 to zmax
             spplot(map, zcol = zcol, main = main,
                    at = seq(0, zmax, length.out = length(col.regions) + 1L),
-                   col.regions = col.regions, sp.layout = sp.layout, ...),
+                   col.regions = col.regions, sp.layout = sp.layout, ...)
+        },
         zcol = names(comps), main = main, zmax = zmax,
         SIMPLIFY = FALSE, USE.NAMES = FALSE)
     if (length(grobs) == 1L) {
