@@ -5,9 +5,9 @@
 ###
 ### Auxiliary functions for operations on spatial data
 ###
-### Copyright (C) 2009-2015,2018 Sebastian Meyer
-### $Revision: 2178 $
-### $Date: 2018-07-17 11:04:22 +0200 (Tue, 17. Jul 2018) $
+### Copyright (C) 2009-2015,2018,2021 Sebastian Meyer
+### $Revision: 2602 $
+### $Date: 2021-01-13 18:36:11 +0100 (Wed, 13. Jan 2021) $
 ################################################################################
 
 
@@ -18,7 +18,7 @@ discpoly <- function (center, radius, npoly = 64,
                       hole = FALSE)
 {
     class <- match.arg(class)
-    if (class == "owin") { # use spatstat::disc
+    if (class == "owin") { # use spatstat.geom::disc
         res <- disc(radius=radius, centre=center, mask=FALSE, npoly=npoly)
         if (hole) {
             res$bdry[[1]]$x <- rev(res$bdry[[1]]$x)
@@ -91,6 +91,13 @@ unionSpatialPolygons <- function (SpP,
 }
 
 
+### internal implementation of as(W, "owin") from polyCub
+### to avoid upgrade problems with polyCub <= 0.7.1 referring to old spatstat
+### and to avoid calling as(W, "owin") from maptools (depends on load order)
+
+SpP2owin <- function (W, ...) owin(poly = xylist(W), ...)
+
+
 ### Compute distance from points to a polygonal boundary
 
 ## since spatstat 1.56-0, bdist.points() interfaces C-code via
